@@ -8,7 +8,7 @@
 # Copyright (C) 2006 Marien Zwart <marienz@gentoo.org>
 # Copyright (C) 2006, 2007 Panoramic Feedback <abentley@panoramicfeedback.com>
 # Copyright (C) 2006,2008,2009 Jelmer Vernooij <jelmer@samba.org>
-# Copyright (C) 2009 Martin von Gagern <Martin.vGagern@gmx.net>
+# Copyright (C) 2009,2010 Martin von Gagern <Martin.vGagern@gmx.net>
 # All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
@@ -483,7 +483,10 @@ class BzrRepository(versioncontrol.Repository):
         for relpath, branch in branches:
             cache = self.get_branch_cache(branch)
             revid = branch.last_revision()
-            rhs = walk(cache, revid)
+            if revid != NULL_REVISION:
+                rhs = walk(cache, revid)
+            else:
+                rhs = []
             rhss.append(rhs)
         for (relpath, branch), rhs in zip(branches, rhss):
             cache = self.get_branch_cache(branch)
@@ -525,6 +528,8 @@ class BzrRepository(versioncontrol.Repository):
                 tree = revbranch.repository.revision_tree(revid)
         except (errors.NoSuchRevision, errors.RevisionNotPresent):
             raise versioncontrol.NoSuchChangeset(rev)
+        if not tree.inventory.root:
+            return EmptyBranchNode(self, branch, path)
         file_id = tree.inventory.path2id(relpath)
         if file_id is None:
             raise versioncontrol.NoSuchNode(path, rev)
@@ -795,8 +800,7 @@ class UnversionedDirNode(BzrNode):
                     node = BzrDirNode(self.bzr_repo, branch, tree,
                                       tree.inventory.root, child_path)
                 else:
-                    node = UnversionedDirNode(self.bzr_repo, self.branch,
-                                              self.revid, child_path)
+                    node = EmptyBranchNode(self.bzr_repo, branch, child_path)
                 result.append(node)
         return result
 
@@ -806,8 +810,11 @@ class UnversionedDirNode(BzrNode):
     def get_content(self):
         return StringIO.StringIO('')
 
+    def get_content_length(self):
+        return None
+
     def get_content_type(self):
-        return 'application/octet-stream'
+        return None
 
     def get_history(self, limit=None):
         if self.branch is not None:
@@ -828,6 +835,37 @@ class UnversionedDirNode(BzrNode):
         return
 
 
+class EmptyBranchNode(BzrNode):
+
+    def __init__(self, bzr_repo, branch, path):
+        rev_string = bzr_repo.string_rev(branch, NULL_REVISION)
+        BzrNode.__init__(self, path, rev_string, versioncontrol.Node.DIRECTORY)
+        self.bzr_repo = bzr_repo
+        self.branch = branch
+        self.path = path
+
+    def __repr__(self):
+        return 'EmptyBranchNode(path=%r)' % self.path
+
+    def get_properties(self):
+        return {}
+
+    def get_entries(self):
+        return []
+
+    def get_content_length(self):
+        return None 
+
+    def get_content(self):
+        return StringIO.StringIO('')
+
+    def get_content_type(self):
+        return None
+
+    def get_history(self, limit=None):
+        return []
+
+
 class BzrVersionedNode(BzrNode):
 
     _diff_map = {
